from pathlib import Path
from typing import Union

class Drive:
    def __init__(self, root: Union[Path, str], create: bool = ...) -> None: ...
    def exists(self, key: str) -> bool: ...
    def save(self, key: str, data: bytes) -> bool: ...
    def load(self, key: str) -> bytes: ...
    def delete(self, key: str) -> bool: ...



import hashlib
from typing import Iterator


def hexdigest(iterator: Iterator[bytes], algorithm) -> str:
    hasher = algorithm()
    for block in iterator:
        hasher.update(block)
    return hasher.hexdigest()


def blake2b(data: bytes, block_size=2 ** 20):
    if not isinstance(data, bytes):
        raise ValueError
    iterator = (data[i : i + block_size] for i in range(0, len(data), block_size))
    return hexdigest(iterator, hashlib.blake2b)


registry = dict(
    blake2b=blake2b,
)


def get_hash_fn(name: str):
    if name not in registry:
        raise ValueError
    return registry[name]



__all__ = ("MegaCloud",)

from ..tool import bytes_to_temp_file

from mega import Mega  # type: ignore
import tempfile


class MegaCloud:
    def __init__(self, email, password, root: str = "redast"):
        mega = Mega()
        self._mega = mega.login(email, password)
        self._root_name = root
        self._root = self._mega.create_folder(root)[root]

    def _path(self, key: str) -> str:
        return self._root_name + "/" + key

    @staticmethod
    def _assert_type_key(key):
        assert isinstance(key, str)

    @staticmethod
    def _assert_type_data(key):
        assert isinstance(key, bytes)

    def exists(self, key: str) -> bool:
        MegaCloud._assert_type_key(key)
        path = self._path(key)
        find = self._mega.find(path)
        if find is None:
            return False
        return True

    def save(self, key: str, data: bytes) -> bool:
        MegaCloud._assert_type_key(key)
        MegaCloud._assert_type_data(data)
        with bytes_to_temp_file(data) as path:
            self._mega.upload(path, self._root, key)
        # FIXME: may not be true
        return True

    def load(self, key: str) -> bytes:
        MegaCloud._assert_type_key(key)
        file_info = self._mega.find(self._path(key))
        with tempfile.TemporaryDirectory() as tempdir:
            temppath = self._mega.download(file_info, dest_path=tempdir)
            with open(temppath, "rb") as file:
                data = file.read()
        return data

    def delete(self, key: str) -> bool:
        MegaCloud._assert_type_key(key)
        file_info = self._mega.find(self._path(key))
        self._mega.delete(file_info[0])
        # FIXME: may not be true
        return True



class MegaCloud:
    def __init__(self, email, password, root: str = ...) -> None: ...
    def exists(self, key: str) -> bool: ...
    def save(self, key: str, data: bytes) -> bool: ...
    def load(self, key: str) -> bytes: ...
    def delete(self, key: str) -> bool: ...



__all__ = ("Drive",)

from pathlib import Path
from typing import Union


class Drive:
    # TODO: split hash by dir parts
    def __init__(self, root: Union[Path, str], create: bool = False):
        root = Path(root)
        if create:
            root.mkdir(mode=0o750, parents=False, exist_ok=True)
        assert root.exists() and root.is_dir()
        self._root = root

    @staticmethod
    def _assert_type_key(key):
        assert isinstance(key, str)

    @staticmethod
    def _assert_type_data(key):
        assert isinstance(key, bytes)

    def exists(self, key: str) -> bool:
        Drive._assert_type_key(key)
        return (self._root / key).exists()

    def save(self, key: str, data: bytes) -> bool:
        Drive._assert_type_key(key)
        Drive._assert_type_data(data)
        try:
            with open(self._root / key, "wb") as file:
                file.write(data)
            return True
        except Exception:
            return False

    def load(self, key: str) -> bytes:
        Drive._assert_type_key(key)
        with open(self._root / key, "rb") as file:
            data = file.read()
        return data

    def delete(self, key: str) -> bool:
        Drive._assert_type_key(key)
        try:
            Path(self._root / key).unlink()
            return True
        except Exception:
            return False



import runpy
from pathlib import Path

from setuptools import find_packages, setup

name = "redast"
short_descriprion = "remote data storage"
author = "Vladislav A. Proskurov"
author_email = "rilshok@pm.me"
url = "https://github.com/rilshok/redast"
license = "MIT"
classifiers = [
    "Development Status :: 2 - Pre-Alpha",
    "Programming Language :: Python :: 3.8",
    "Programming Language :: Python :: 3.9",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",
]


version_path = Path(__file__).resolve().parent / name / "__version__.py"
version = runpy.run_path(version_path)["__version__"]

with open("README.md", "r") as file:
    long_description = file.read()

with open("requirements.txt", encoding="utf-8") as file:
    requirements = file.read().splitlines()

setup(
    name=name,
    author=author,
    author_email=author_email,
    descriprion=short_descriprion,
    long_description=long_description,
    long_description_content_type="text/markdown",
    license=license,
    url=url,
    packages=find_packages(include=(name,)),
    include_package_data=True,
    version=version,
    install_requires=requirements,
    classifiers=classifiers,
    python_requires=">=3.8",
)
